# coding=utf-8
"""
测试扫单性能
"""
import json
import logging
import os
import sqlite3
import sys
import time
import uuid
import random
from datetime import datetime

symbols = [
    "SHSE.600000", "SHSE.600004", "SHSE.600006", "SHSE.600007", "SHSE.600008", "SHSE.600009", "SHSE.600010",
    "SHSE.600012", "SHSE.600015", "SHSE.600016", "SHSE.600017", "SHSE.600018", "SHSE.600019", "SHSE.600020",
    "SHSE.600022", "SHSE.600023", "SHSE.600025", "SHSE.600026", "SHSE.600027", "SHSE.600028", "SHSE.600029",
    "SHSE.600031", "SHSE.600033", "SHSE.600035", "SHSE.600036", "SHSE.600037", "SHSE.600038", "SHSE.600039",
    "SHSE.600050", "SHSE.600051", "SHSE.600052", "SHSE.600053", "SHSE.600054", "SHSE.600055", "SHSE.600056",
    "SHSE.600058", "SHSE.600059", "SHSE.600060", "SHSE.600061", "SHSE.600062", "SHSE.600063", "SHSE.600064",
    "SHSE.600067", "SHSE.600068", "SHSE.600069", "SHSE.600070", "SHSE.600071", "SHSE.600072", "SHSE.600073",
    "SHSE.600076", "SHSE.600077", "SHSE.600078", "SHSE.600079", "SHSE.600080", "SHSE.600081", "SHSE.600082",
    "SHSE.600084", "SHSE.600085", "SHSE.600086", "SHSE.600088", "SHSE.600089", "SHSE.600090", "SHSE.600093",
    "SHSE.600095", "SHSE.600096", "SHSE.600097", "SHSE.600098", "SHSE.600099", "SHSE.600100", "SHSE.600101",
    "SHSE.600104", "SHSE.600105", "SHSE.600106", "SHSE.600107", "SHSE.600108", "SHSE.600109", "SHSE.600110",
    "SHSE.600112", "SHSE.600113", "SHSE.600114", "SHSE.600115", "SHSE.600116", "SHSE.600117", "SHSE.600118",
    "SHSE.600120", "SHSE.600121", "SHSE.600122", "SHSE.600123", "SHSE.600125", "SHSE.600126", "SHSE.600127",
    "SHSE.600129", "SHSE.600130", "SHSE.600131", "SHSE.600132", "SHSE.600133", "SHSE.600135", "SHSE.600136",
]

logging.basicConfig(stream=sys.stdout, level=logging.INFO,
                    format='%(asctime)-8s [%(threadName)s] %(message)s')

indb = sqlite3.connect(os.path.join(os.path.dirname(__file__), 'in.db'), check_same_thread=False)
outdb = sqlite3.connect(os.path.join(os.path.dirname(__file__), 'out.db'), check_same_thread=False)

sys.setrecursionlimit(1000000)


def _maxidval(sql: str) -> int:
    cur = outdb.cursor()
    cur.execute(sql)
    row = cur.fetchone()
    if row:
        return row[0]
    else:
        return 0


execrpt_id = _maxidval('select max(id) as maxid from out_execrpt')

order_id = _maxidval('select max(id) as maxid from out_order')

# 0.01s = 10ms
loop_sleep_time = 0.01


def _fetchrows(sql: str):
    try:
        cur = outdb.cursor()
        cur.execute(sql)
        rows = cur.fetchall()
        return rows
    except Exception as e:
        print(e, "重试执行")
        return _fetchrows(sql)


def _fetchrows_withid(sql: str, idval: int):
    try:
        cur = outdb.cursor()
        cur.execute(sql, [idval])
        rows = cur.fetchall()
        return rows
    except Exception as e:
        print(e, "重试执行")
        return _fetchrows_withid(sql, idval)


def place_order():
    # 指定的symbol, 按市价委托, 100，其中，sid是用户需要维护的订单标识
    sid_dict = {}
    sid = uuid.uuid4().hex
    sid_dict[sid] = [datetime.now()]
    symbol = random.choice(symbols)
    order_business = 0  # 委托业务属性，0表示普通交易，否则为指定的两融等委托业务
    volume = 100
    # logging.info("下单 sid=%s, symbol=%s, volume=%s", sid, symbol, volume)
    sql = f"""
        insert into in_place_order (sid, symbol, volume, price, side, position_effect, order_type, order_duration, order_qualifier, order_business)
        values ('{sid}', '{symbol}', '{volume}', 0, 1, 1, 2, 0, 0, {order_business})
    """
    try:
        cur = indb.cursor()
        cur.execute(sql)
        indb.commit()
    except Exception as e:
        print(e)
    return sid


def place_orders(num):
    """
    批量委托
    :param num:
    :return:
    """
    sids = []
    # 指定的symbol, 按市价委托, 100，其中，sid是用户需要维护的订单标识
    order_business = 0  # 委托业务属性，0表示普通交易，否则为指定的两融等委托业务
    volume = 100
    # logging.info("下单 sid=%s, symbol=%s, volume=%s", sid, symbol, volume)
    sql = """insert into in_place_order (sid, symbol, volume, price, side, position_effect, order_type, order_duration, order_qualifier, order_business) values """
    sql_values = []
    for i in range(num):
        sid = uuid.uuid4().hex
        sids.append(sid)
        symbol = random.choice(symbols)
        sql_values.append(f"""('{sid}', '{symbol}', '{volume}', 0, 1, 1, 2, 0, 0, {order_business})""")

    sql = sql + ",".join(sql_values)
    cur = indb.cursor()
    cur.execute(sql)
    indb.commit()
    return sids


def get_order(sid, loop_times=0):
    """
    查询已报委托
    :param sid:
    :param loop_times:
    :return:
    """
    sql = f"""
    select * from out_order where sid = ? and status =1
    """
    rows = _fetchrows_withid(sql, sid)
    if len(rows) == 0:
        time.sleep(loop_sleep_time)
        loop_times += 1
        return get_order(sid, loop_times)
    return rows[0], loop_times


def get_orders(sids, loop_times=0):
    """
    查询已报委托
    :param sids:
    :param loop_times:
    :return:
    """
    if len(sids) == 0:
        return
    cond_sids = sids[:]
    res = {}
    sids_in = '"' + '","'.join(cond_sids) + '"'
    sql = f"""
    select * from out_order
    where sid in ({sids_in}) and status =1
    """
    rows = _fetchrows(sql)
    if len(rows) > 0:
        for row in rows:
            res[row[1]] = datetime.now()
            cond_sids.remove(row[1])

    if len(cond_sids) > 0:
        time.sleep(loop_sleep_time)
        loop_times += 1
        res.update(get_orders(cond_sids, loop_times))
    return res

def count_spend_time(end_time, start_time):
    delta = end_time - start_time
    return delta.seconds * 1000 + delta.microseconds / 1000


def perf_order(place_total=100, place_interval=2):
    if place_total <= 0:
        return
    total_spend = 0
    for i in range(place_total):
        start = datetime.now()
        sid = place_order()
        write_db_spend = count_spend_time(datetime.now(), start)
        get_order(sid)
        end = datetime.now()
        spend = count_spend_time(end, start)
        d={"index": i+1, "start": f"{start}", "end": f"{end}", "write_spend": write_db_spend, "spend": spend}
        print(json.dumps(d))
        total_spend += spend
        time.sleep(place_interval)
    print(f"\n逐笔委托 {place_total} 笔，每笔委托间隔 {place_interval} 秒，平均每笔委托从报单到查询到委托已报耗时 {total_spend / place_total} 毫秒")


def perf_orders(place_total=100):
    start = datetime.now()
    print("开始委托：", start)
    sids = place_orders(num=place_total)
    write_end = datetime.now()
    print("入库完成：", write_end)
    print("入库耗时：", write_end - start)
    rows = get_orders(sids)
    total_spend = 0
    for sid in sids:
        end = rows.get(sid)
        delta = end - start
        spend = delta.seconds * 1000 + delta.microseconds / 1000
        total_spend+=spend
        d={"sid": sid, "start": f"{start}", "end": f"{end}", "spend": spend}
        print(json.dumps(d))
    end = datetime.now()
    delta = end - start
    spend = delta.seconds * 1000 + delta.microseconds / 1000
    print(f"\n全部结束：{end}，批量委托 {place_total} 笔，总耗时：{spend}。 平均每笔委托从报单到查询到委托已报耗时 {total_spend / place_total} 毫秒")


if __name__ == '__main__':
    # 下1000笔，每笔委托间隔0秒
    perf_order(place_total=1000, place_interval=0)

    # 批量委托
    # perf_orders(place_total=1000)
